# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winui3._winui3_microsoft_windows_storage import (
    ApplicationData,
    ApplicationDataContainer,
)

__all__ = [
    "ApplicationDataCreateDisposition",
    "ApplicationDataLocality",
    "ApplicationData",
    "ApplicationDataContainer",
]

class ApplicationDataCreateDisposition(enum.IntEnum):
    ALWAYS = 0
    EXISTING = 1

class ApplicationDataLocality(enum.IntEnum):
    LOCAL = 0
    LOCAL_CACHE = 3
    SHARED_LOCAL = 4
    TEMPORARY = 2
    MACHINE = 1000

